/*

 partition.c - A mostly working PCLink server for IDEDOS 0.9x

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "partition.h"
#include <string.h>
#include <stdlib.h>

static partition_t work_partition;

static struct partition_s {
    c64c_t *name[17];
    char *path;
} partitions[256];

void partition_create(partition_t n, const c64c_t *name) {
    struct partition_s *p = &partitions[n];

    strncpy((char *)(p->name), (char *)name, 16);
    p->name[16]=0;
    p->path = malloc(1);
    p->path[0]=0;
}

int partition_select(partition_t n) {
    struct partition_s *p = &partitions[n];

    if (!p->path) return 77;
    work_partition = n;
    return 2;
}

char *partition_get_path(partition_t n) {
    return partitions[n ? n : work_partition].path;
}

partition_t partition_get_current(void) {
    return work_partition;
}

c64c_t *partition_get_name(partition_t n) {
    struct partition_s *p = &partitions[n ? n : work_partition];

    return (p->path != NULL) ? (c64c_t *)p->name : (c64c_t *)NULL;
}

void partition_set_path(partition_t n, const char *path) {
    struct partition_s *p;
    p = &partitions[n ? n : work_partition];
    
    if (p->path) {
        if (strlen(path) > strlen(p->path)) p->path = realloc(p->path, strlen(path) + 1);
        strcpy(p->path, path);
    }
}
